#version 130
#extension GL_EXT_gpu_shader3 : enable
#extension GL_EXT_gpu_shader4 : enable

//##COMMON_DEFINITIONS_SHADER##

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int useTexAlpha;
uniform int dissolveMode;

attribute vec2 colorCurveST;
attribute float filterAlpha;
attribute int paramBit;

uniform int lightNumber;
uniform int lightRecieve[7];

out vec3 position;
out vec3 normal;
out vec4 color;
out vec4 colorDiffuse;
out vec4 colorSpecular;
out vec4 colorDiffuseTotal;
out vec4 colorSpecularTotal;

flat out int blend;
flat out int bFlat;
flat out int textureImageIndex; // valid 0-3 not use if 4;

//##FUNC_SPOT_EFFECT##

void main(void)
{
    gl_TexCoord[0] = gl_TextureMatrix[0] * gl_MultiTexCoord0;
	
    vec3 position_vertex = vec3(gl_ModelViewMatrix * gl_Vertex);
    vec3 normal = normalize(gl_NormalMatrix * gl_Normal);

    colorDiffuseTotal = vec4(0, 0, 0, 1);
    colorSpecularTotal = vec4(0, 0, 0, 1);
    
    vec3 viewVec = - normalize(position_vertex);
    
    vec4 baseColor = texture2D(textureColorCurve, colorCurveST);
    baseColor[3] *= filterAlpha;

    vec4 colorDiffuseBase = texture2D(textureColorCurve, colorCurveST+vec2(0, 0.3333));
    vec4 colorSpecularBase = texture2D(textureColorCurve, colorCurveST+vec2(0, 0.6666));
    
    float alphaOrg = baseColor.a;
    float alpha0 = baseColor[3];

    if (dissolveMode == 1)
	if (alphaOrg < 0.015)
	    alpha0 = 1;
    
    if (dissolveMode == 2)
	alpha0 = 1;
    
    color = vec4(baseColor.rgb * alpha0, alphaOrg);
    colorDiffuse = vec4(colorDiffuseBase.rgb * alpha0, alphaOrg);
    colorSpecular = vec4(colorSpecularBase.rgb * alpha0, alphaOrg);

    float shininess = 20;
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;

	vec3 lightVec = gl_LightSource[i].position.xyz - position_vertex * gl_LightSource[i].position[3];
	
	float dis = length(lightVec);
	
	//float attenuation = 1.0;
	
	lightVec = normalize(lightVec);

	float diffuse = dot(lightVec, normal);	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    lightFactor /= (gl_LightSource[0].constantAttenuation +
			    gl_LightSource[0].linearAttenuation * dis +
			    gl_LightSource[0].quadraticAttenuation * dis * dis);
	    
	    vec3 reflectVec = reflect(-lightVec, normal);
	    float specular = pow(max(dot(viewVec, reflectVec), 0.0), 
				 /*gl_FrontMaterial.shininess*/shininess);
	    colorDiffuseTotal += diffuse * vec4(gl_LightSource[i].diffuse.rgb, 0) * lightFactor;
	    colorSpecularTotal += specular * vec4(gl_LightSource[i].specular.rgb, 0);
	}
    }
    
    position = vec3(gl_ModelViewMatrix * gl_Vertex);
    //normal = gl_NormalMatrix * gl_Normal;

    gl_Position = ftransform();

    blend = paramBit % _PARAMBIT_BLEND_MOD;

    bFlat = 0; //!!

    if ((paramBit / _PARAMBIT_USE_TEXTURE) % 2 == 0)
	textureImageIndex = 4;
    else
	textureImageIndex = (paramBit / _PARAMBIT_TEXTUREINDEX) % _PARAMBIT_TEXTUREINDEX_MOD;
}
